#!/bin/bash

export COREMASK=`taskset -p $$ | sed 's/^[^:]*://' | awk '{ print "0x" $1 }'`
export CLFS=${PWD}/sysroot
export SU_CMD="sudo"
export CLFS_HOST=$(echo ${MACHTYPE} | sed -e 's/-[^-]*/-cross/')
export CLFS_TARGET=armv7l-clfs-linux-gnueabihf
export CLFS_TARBALL_DIR=${PWD}/tarballs
export CLFS_USER=`id -un`

PATH=/cross-tools/bin:$PATH

for (( CORECOUNT=0; COREMASK>0; COREMASK>>=1 )); do
    if [ $((COREMASK&0x1)) -ne 0 ]; then
        CORECOUNT=$((CORECOUNT+1))
    fi
done

mkdir -p ${CLFS} ${CLFS_TARBALL_DIR}
[ ! -h /tools ] || $SU_CMD rm -f /tools || exit 1
install -dv ${CLFS}/tools
$SU_CMD ln -sv ${CLFS}/tools /
install -dv ${CLFS}/cross-tools
[ ! -h /cross-tools ] || $SU_CMD rm -f /cross-tools || exit 1
$SU_CMD ln -sv ${CLFS}/cross-tools /

mkdir -pv ${CLFS}/dev &&
$SU_CMD mknod -m 0600 ${CLFS}/dev/mmcblk0 b 179 0 &&
$SU_CMD mknod -m 0600 ${CLFS}/dev/mmcblk0p1 b 179 1 &&
$SU_CMD mknod -m 0600 ${CLFS}/dev/mmcblk0p2 b 179 2 &&
$SU_CMD mknod -m 0600 ${CLFS}/dev/console c 5 1 &&
$SU_CMD mknod -m 0666 ${CLFS}/dev/null c 1 3 ||
exit $?

FILE_VER=5.18
FILE_URL="ftp://ftp.astron.com/pub/file/file-${FILE_VER}.tar.gz"
FILE_NAME="${FILE_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${FILE_NAME} ] || wget ${FILE_URL} -O ${CLFS_TARBALL_DIR}/${FILE_NAME} || exit 1
echo "BUILDING file-${FILE_VER}..."
rm -Rf file-${FILE_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${FILE_NAME} &&
cd file-${FILE_VER} &&
./configure --prefix=/cross-tools --disable-static &&
make -j $CORECOUNT -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

BC_VER=1.06.95
BC_URL="ftp://alpha.gnu.org/gnu/bc/bc-${BC_VER}.tar.bz2"
BC_NAME="${BC_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${BC_NAME} ] || wget ${BC_URL} -O ${CLFS_TARBALL_DIR}/${BC_NAME} || exit 1
echo "BUILDING bc-${BC_VER}..."
rm -Rf bc-${BC_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${BC_NAME} &&
cd bc-${BC_VER} &&
./configure --prefix=/cross-tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

LINUX_VER=3.12.0
LINUX_URL="https://github.com/parallella/parallella-linux-adi/archive/xcomm_zynq.tar.gz"
LINUX_NAME=linux-parallella-${LINUX_VER}.tar.gz
[ -f ${CLFS_TARBALL_DIR}/${LINUX_NAME} ] || wget ${LINUX_URL} -O ${CLFS_TARBALL_DIR}/${LINUX_NAME} || exit 1
echo "BUILDING kernel-headers-${LINUX_VER}..."
rm -Rf parallella-linux-adi-xcomm_zynq &&
tar xvf ${CLFS_TARBALL_DIR}/${LINUX_NAME} &&
cd parallella-linux-adi-xcomm_zynq &&
make mrproper &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- headers_check &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- INSTALL_HDR_PATH=/tools headers_install &&
cd .. ||
exit $?

M4_VER=1.4.17
M4_URL="http://ftp.gnu.org/gnu/m4/m4-${M4_VER}.tar.bz2"
M4_NAME="${M4_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${M4_NAME} ] || wget ${M4_URL} -O ${CLFS_TARBALL_DIR}/${M4_NAME} || exit 1
echo "BUILDING m4-${M4_VER}..."
rm -Rf m4-${M4_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${M4_NAME} &&
cd m4-${M4_VER} &&
./configure --prefix=/cross-tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

NCURSES_VER=5.9
NCURSES_URL="ftp://ftp.gnu.org/pub/gnu/ncurses/ncurses-${NCURSES_VER}.tar.gz"
NCURSES_NAME="${NCURSES_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${NCURSES_NAME} ] || wget ${NCURSES_URL} -O ${CLFS_TARBALL_DIR}/${NCURSES_NAME} || exit 1
NCURSES_PATCH_URL=http://patches.cross-lfs.org/dev/ncurses-${NCURSES_VER}-bash_fix-1.patch
NCURSES_PATCH_NAME="${NCURSES_PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${NCURSES_PATCH_NAME} ] || wget ${NCURSES_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${NCURSES_PATCH_NAME} || exit 1
echo "BUILDING ncurses-${NCURSES_VER}..."
rm -Rf ncurses-${NCURSES_VER}
tar xvf ${CLFS_TARBALL_DIR}/${NCURSES_NAME} &&
cd ncurses-${NCURSES_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${NCURSES_PATCH_NAME} &&
./configure --prefix=/cross-tools --without-debug --without-shared &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 -C include &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 -C progs tic &&
install -v -m755 progs/tic /cross-tools/bin &&
cd .. ||
exit $?

PKG_CONFIG_LITE_VER=0.28-1
PKG_CONFIG_LITE_URL="http://sourceforge.net/projects/pkgconfiglite/files/${PKG_CONFIG_LITE_VER}/pkg-config-lite-${PKG_CONFIG_LITE_VER}.tar.gz"
PKG_CONFIG_LITE_NAME="${PKG_CONFIG_LITE_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${PKG_CONFIG_LITE_NAME} ] || wget ${PKG_CONFIG_LITE_URL} -O ${CLFS_TARBALL_DIR}/${PKG_CONFIG_LITE_NAME} || exit 1
echo "BUILDING pkg-config-lite-${PKG_CONFIG_LITE_VER}..."
rm -Rf pkg-config-lite-${PKG_CONFIG_LITE_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${PKG_CONFIG_LITE_NAME} &&
cd pkg-config-lite-${PKG_CONFIG_LITE_VER} &&
./configure --prefix=/cross-tools --with-pc-path=/tools/lib/pkgconfig:/tools/share/pkgconfig &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

GMP_VER=6.0.0
GMP_URL="http://ftp.gnu.org/gnu/gmp/gmp-${GMP_VER}a.tar.bz2"
GMP_NAME="${GMP_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GMP_NAME} ] || wget ${GMP_URL} -O ${CLFS_TARBALL_DIR}/${GMP_NAME} || exit 1
echo "BUILDING gmp-${GMP_VER}..."
rm -Rf gmp-${GMP_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GMP_NAME} &&
cd gmp-${GMP_VER} &&
./configure --prefix=/cross-tools --enable-cxx --disable-static &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

MPFR_VER=3.1.2
MPFR_URL="http://www.mpfr.org/mpfr-current/mpfr-${MPFR_VER}.tar.bz2"
MPFR_NAME="${MPFR_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${MPFR_NAME} ] || wget ${MPFR_URL} -O ${CLFS_TARBALL_DIR}/${MPFR_NAME} || exit 1
MPFR_PATCH_URL="http://patches.cross-lfs.org/dev/mpfr-${MPFR_VER}-fixes-3.patch"
MPFR_PATCH_NAME="${MPFR_PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${MPFR_PATCH_NAME} ] || wget ${MPFR_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${MPFR_PATCH_NAME} || exit 1
echo "BUILDING mpfr-${MPFR_VER}..."
rm -Rf mpfr-${MPFR_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${MPFR_NAME} &&
cd mpfr-${MPFR_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${MPFR_PATCH_NAME} &&
LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
./configure --prefix=/cross-tools --disable-static --with-gmp=/cross-tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

MPC_VER=1.0.2
MPC_URL="ftp://ftp.gnu.org/gnu/mpc/mpc-${MPC_VER}.tar.gz"
MPC_NAME="${MPC_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${MPC_NAME} ] || wget ${MPC_URL} -O ${CLFS_TARBALL_DIR}/${MPC_NAME} || exit 1
echo "BUILDING mpc-${MPC_VER}..."
rm -Rf mpc-${MPC_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${MPC_NAME} &&
cd mpc-${MPC_VER} &&
LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
./configure --prefix=/cross-tools --disable-static --with-gmp=/cross-tools --with-mpfr=/cross-tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

ISL_VER=0.12.2
ISL_URL="http://isl.gforge.inria.fr/isl-${ISL_VER}.tar.bz2"
ISL_NAME="${ISL_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${ISL_NAME} ] || wget ${ISL_URL} -O ${CLFS_TARBALL_DIR}/${ISL_NAME} || exit 1
echo "BUILDING isl-${ISL_VER}..."
rm -Rf isl-${ISL_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${ISL_NAME} &&
cd isl-${ISL_VER} &&
LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
./configure --prefix=/cross-tools --disable-static --with-gmp-prefix=/cross-tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

CLOOG_VER=0.18.2
CLOOG_URL="http://www.bastoul.net/cloog/pages/download/cloog-${CLOOG_VER}.tar.gz"
CLOOG_NAME="${CLOOG_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${CLOOG_NAME} ] || wget ${CLOOG_URL} -O ${CLFS_TARBALL_DIR}/${CLOOG_NAME} || exit 1
echo "BUILDING CLooG-${CLOOG_VER}..."
rm -Rf cloog-${CLOOG_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${CLOOG_NAME} &&
cd cloog-${CLOOG_VER} &&
LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
./configure --prefix=/cross-tools --disable-static --with-gmp-prefix=/cross-tools --with-isl-prefix=/cross-tools &&
cp -v Makefile{,.orig} &&
sed '/cmake/d' Makefile.orig > Makefile &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

BINUTILS_VER=2.24
BINUTILS_URL="http://ftp.gnu.org/gnu/binutils/binutils-${BINUTILS_VER}.tar.bz2"
BINUTILS_NAME="${BINUTILS_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${BINUTILS_NAME} ] || wget ${BINUTILS_URL} -O ${CLFS_TARBALL_DIR}/${BINUTILS_NAME} || exit 1
echo "BUILDING cross-binutils-${BINUTILS_VER}..."
rm -Rf binutils-${BINUTILS_VER} binutils-build &&
tar xvf ${CLFS_TARBALL_DIR}/${BINUTILS_NAME} &&
mkdir -v binutils-build &&
cd binutils-build &&
AR=ar AS=as \
../binutils-${BINUTILS_VER}/configure \
    --prefix=/cross-tools --host=${CLFS_HOST} --target=${CLFS_TARGET} \
    --with-sysroot=${CLFS} --with-lib-path=/tools/lib --disable-nls \
    --disable-static --disable-multilib --disable-werror &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

GCC_VER=4.8.3
GCC_URL="ftp://gcc.gnu.org/pub/gcc/releases/gcc-${GCC_VER}/gcc-${GCC_VER}.tar.bz2"
GCC_NAME="${GCC_URL##*/}"
GCC_SPECS_PATCH_URL="http://patches.cross-lfs.org/dev/gcc-${GCC_VER}-specs-1.patch"
GCC_SPECS_PATCH_NAME="${GCC_SPECS_PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GCC_NAME} ] || wget ${GCC_URL} -O ${CLFS_TARBALL_DIR}/${GCC_NAME} || exit 1
echo "BUILDING cross-gcc-static-${GCC_VER}..."
rm -Rf gcc-${GCC_VER} gcc-build &&
tar xvf ${CLFS_TARBALL_DIR}/${GCC_NAME} &&
cd gcc-${GCC_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${GCC_SPECS_PATCH_NAME} &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_1\n#define STANDARD_STARTFILE_PREFIX_1 "/tools/lib/"\n' >> gcc/config/linux.h &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_2\n#define STANDARD_STARTFILE_PREFIX_2 ""\n' >> gcc/config/linux.h &&
touch /tools/include/limits.h &&
mkdir -v ../gcc-build &&
cd ../gcc-build &&
AR=ar LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
../gcc-${GCC_VER}/configure --prefix=/cross-tools \
    --build=${CLFS_HOST} --host=${CLFS_HOST} --target=${CLFS_TARGET} \
    --with-sysroot=${CLFS} --with-local-prefix=/tools \
    --with-native-system-header-dir=/tools/include --disable-nls \
    --disable-shared --with-mpfr=/cross-tools --with-gmp=/cross-tools \
    --with-isl=/cross-tools --with-cloog=/cross-tools --with-mpc=/cross-tools \
    --without-headers --with-newlib --disable-decimal-float --disable-libgomp \
    --disable-libmudflap --disable-libssp --disable-libatomic --disable-libitm \
    --disable-libsanitizer --disable-libquadmath --disable-threads \
    --disable-multilib --disable-target-zlib --with-system-zlib \
    --enable-languages=c --enable-checking=release &&
#make -j ${CORECOUNT} -l ${CORECOUNT}.0 all-gcc all-target-libgcc &&
make all-gcc all-target-libgcc &&
make install-gcc install-target-libgcc &&
cp -v ../gcc-${GCC_VER}/include/libiberty.h /tools/include &&
cd .. ||
exit $?

GLIBC_VER=2.19
GLIBC_URL="ftp://ftp.gnu.org/pub/gnu/glibc/glibc-${GLIBC_VER}.tar.bz2"
GLIBC_NAME="${GLIBC_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GLIBC_NAME} ] || wget ${GLIBC_URL} -O ${CLFS_TARBALL_DIR}/${GLIBC_NAME} || exit 1
echo "BUILDING glibc-${GLIBC_VER}..."
rm -Rf glibc-${GLIBC_VER} glibc-build &&
tar xvf ${CLFS_TARBALL_DIR}/${GLIBC_NAME} &&
cd glibc-${GLIBC_VER} &&
cp -v timezone/Makefile{,.orig} &&
sed 's/\\$$(pwd)/`pwd`/' timezone/Makefile.orig > timezone/Makefile &&
mkdir -v ../glibc-build &&
cd ../glibc-build &&
echo 'libc_cv_ssp=no' > config.cache &&
BUILD_CC="gcc" CC="${CLFS_TARGET}-gcc" \
AR="${CLFS_TARGET}-ar" RANLIB="${CLFS_TARGET}-ranlib" \
../glibc-${GLIBC_VER}/configure --prefix=/tools \
    --host=${CLFS_TARGET} --build=${CLFS_HOST} \
    --disable-profile --with-tls --enable-kernel=2.6.32 \
    --with-__thread --with-binutils=/cross-tools/bin \
    --with-headers=/tools/include --enable-obsolete-rpc \
    --cache-file=config.cache &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING cross-gcc-final-${GCC_VER}..."
rm -Rf gcc-${GCC_VER} gcc-build &&
tar xvf ${CLFS_TARBALL_DIR}/${GCC_NAME} &&
cd gcc-${GCC_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${GCC_SPECS_PATCH_NAME} &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_1\n#define STANDARD_STARTFILE_PREFIX_1 "/tools/lib/"\n' >> gcc/config/linux.h &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_2\n#define STANDARD_STARTFILE_PREFIX_2 ""\n' >> gcc/config/linux.h &&
touch /tools/include/limits.h &&
mkdir -v ../gcc-build &&
cd ../gcc-build &&
AR=ar LDFLAGS="-Wl,-rpath,/cross-tools/lib" \
../gcc-${GCC_VER}/configure --prefix=/cross-tools \
    --build=${CLFS_HOST} --target=${CLFS_TARGET} --host=${CLFS_HOST} \
    --with-sysroot=${CLFS} --with-local-prefix=/tools \
    --with-native-system-header-dir=/tools/include --disable-nls \
    --enable-shared --disable-static --enable-languages=c,c++ \
    --enable-__cxa_atexit --enable-c99 \
    --enable-long-long --enable-threads=posix --disable-multilib \
    --with-mpc=/cross-tools --with-mpfr=/cross-tools --with-gmp=/cross-tools \
    --with-cloog=/cross-tools --with-isl=/cross-tools --with-system-zlib \
    --enable-checking=release --enable-libstdcxx-time &&
#make -j ${CORECOUNT} -l ${CORECOUNT}.0 AS_FOR_TARGET="${CLFS_TARGET}-as" LD_FOR_TARGET="${CLFS_TARGET}-ld" &&
make AS_FOR_TARGET="${CLFS_TARGET}-as" LD_FOR_TARGET="${CLFS_TARGET}-ld" &&
make install &&
cd .. ||
exit $?

export CC="${CLFS_TARGET}-gcc"
export CXX="${CLFS_TARGET}-g++"
export AR="${CLFS_TARGET}-ar"
export AS="${CLFS_TARGET}-as"
export RANLIB="${CLFS_TARGET}-ranlib"
export LD="${CLFS_TARGET}-ld"
export STRIP="${CLFS_TARGET}-strip"

echo "*** CONSTRUCTING TEMPORARY SYSTEM ***"
echo "BUILDING gmp-${GMP_VER}..."
rm -Rf gmp-${GMP_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GMP_NAME} &&
cd gmp-${GMP_VER} &&
CC_FOR_BUILD=gcc ./configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --enable-cxx &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING mpfr-${MPFR_VER}..."
rm -Rf mpfr-${MPFR_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${MPFR_NAME} &&
cd mpfr-${MPFR_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${MPFR_PATCH_NAME} &&
./configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --enable-shared &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING mpc-${MPC_VER}..."
rm -Rf mpc-${MPC_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${MPC_NAME} &&
cd mpc-${MPC_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING isl-${ISL_VER}..."
rm -Rf isl-${ISL_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${ISL_NAME} &&
cd isl-${ISL_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING CLooG-${CLOOG_VER}..."
rm -Rf cloog-${CLOOG_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${CLOOG_NAME} &&
cd cloog-${CLOOG_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
cp -v Makefile{,.orig} &&
sed '/cmake/d' Makefile.orig > Makefile &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

ZLIB_VER=1.2.8
ZLIB_URL="http://zlib.net/zlib-${ZLIB_VER}.tar.gz"
ZLIB_NAME="${ZLIB_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${ZLIB_NAME} ] || wget ${ZLIB_URL} -O ${CLFS_TARBALL_DIR}/${ZLIB_NAME} || exit 1
echo "BUILDING zlib-${ZLIB_VER}..."
rm -Rf zlib-${ZLIB_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${ZLIB_NAME} &&
cd zlib-${ZLIB_VER} &&
./configure --prefix=/tools &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING binutils-${BINUTILS_VER}..."
rm -Rf binutils-${BINUTILS_VER} binutils-build &&
tar xvf ${CLFS_TARBALL_DIR}/${BINUTILS_NAME} &&
mkdir -v binutils-build &&
cd binutils-build &&
../binutils-${BINUTILS_VER}/configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} --target=${CLFS_TARGET} \
    --with-lib-path=/tools/lib --disable-nls --enable-shared \
    --disable-multilib &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING gcc-${GCC_VER}..."
rm -Rf gcc-${GCC_VER} gcc-build &&
tar xvf ${CLFS_TARBALL_DIR}/${GCC_NAME} &&
cd gcc-${GCC_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${GCC_SPECS_PATCH_NAME} &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_1\n#define STANDARD_STARTFILE_PREFIX_1 "/tools/lib/"\n' >> gcc/config/linux.h &&
echo -en '\n#undef STANDARD_STARTFILE_PREFIX_2\n#define STANDARD_STARTFILE_PREFIX_2 ""\n' >> gcc/config/linux.h &&
sed -i 's@\./fixinc\.sh@-c true@' gcc/Makefile.in &&
mkdir -v ../gcc-build &&
cd ../gcc-build &&
../gcc-${GCC_VER}/configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} --target=${CLFS_TARGET} \
    --with-local-prefix=/tools --enable-long-long --enable-c99 --enable-shared \
    --enable-threads=posix --disable-multilib --disable-nls \
    --enable-__cxa_atexit --enable-languages=c,c++ --disable-libstdcxx-pch \
    --with-system-zlib --with-native-system-header-dir=/tools/include \
    --disable-libssp --enable-checking=release --enable-libstdcxx-time &&
cp -v Makefile{,.orig} &&
sed "/^HOST_\(GMP\|ISL\|CLOOG\)\(LIBS\|INC\)/s:/tools:/cross-tools:g" Makefile.orig > Makefile &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 AS_FOR_TARGET="${AS}" LD_FOR_TARGET="${LD}" &&
#make AS_FOR_TARGET="${AS}" LD_FOR_TARGET="${LD}" &&
make install &&
cd .. ||
exit $?


echo "BUILDING ncurses-${NCURSES_VER}..."
rm -Rf ncurses-${NCURSES_VER}
tar xvf ${CLFS_TARBALL_DIR}/${NCURSES_NAME} &&
cd ncurses-${NCURSES_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${NCURSES_PATCH_NAME} &&
./configure --prefix=/tools --with-shared \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --without-debug --without-ada \
    --enable-overwrite --with-build-cc=gcc &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

BASH_VER=4.3
BASH_URL="http://ftp.gnu.org/gnu/bash/bash-${BASH_VER}.tar.gz"
BASH_NAME="${BASH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${BASH_NAME} ] || wget ${BASH_URL} -O ${CLFS_TARBALL_DIR}/${BASH_NAME} || exit 1

for BASH_PATCH_NUM in `seq 18`; do
    BASH_PATCH_NUM=`printf "%03d" $BASH_PATCH_NUM`
    BASH_PATCH_URL="${BASH_URL%/*}/bash-${BASH_VER}-patches/bash${BASH_VER//./}-${BASH_PATCH_NUM}"
    BASH_PATCH_NAME="${BASH_PATCH_URL##*/}"
    [ -f ${CLFS_TARBALL_DIR}/${BASH_PATCH_NAME} ] || wget ${BASH_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${BASH_PATCH_NAME} || exit 1
done

echo "BUILDING bash-${BASH_VER}..."
rm -Rf bash-${BASH_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${BASH_NAME} &&
cd bash-${BASH_VER}

for BASH_PATCH_NUM in `seq 18`; do
    BASH_PATCH_NUM=`printf "%03d" $BASH_PATCH_NUM`
    BASH_PATCH_NAME="bash${BASH_VER//./}-${BASH_PATCH_NUM}"
    echo "Applying patch ${BASH_PATCH_NAME}..."
    patch -p0 < ${CLFS_TARBALL_DIR}/${BASH_PATCH_NAME} || exit 1
done

cat > config.cache << "EOF"
ac_cv_func_mmap_fixed_mapped=yes
ac_cv_func_strcoll_works=yes
ac_cv_func_working_mktime=yes
bash_cv_func_sigsetjmp=present
bash_cv_getcwd_malloc=yes
bash_cv_job_control_missing=present
bash_cv_printf_a_format=yes
bash_cv_sys_named_pipes=present
bash_cv_ulimit_maxfds=yes
bash_cv_under_sys_siglist=yes
bash_cv_unusable_rtsigs=no
gt_cv_int_divbyzero_sigfpe=yes
EOF

./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --without-bash-malloc --cache-file=config.cache &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit 1

BZIP2_VER=1.0.6
BZIP2_URL="http://www.bzip.org/${BZIP2_VER}/bzip2-${BZIP2_VER}.tar.gz"
BZIP2_NAME="${BZIP2_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${BZIP2_NAME} ] || wget ${BZIP2_URL} -O ${CLFS_TARBALL_DIR}/${BZIP2_NAME} || exit 1
echo "BUILDING bzip2-${BZIP2_VER}..."
rm -Rf bzip2-${BZIP2_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${BZIP2_NAME} &&
cd bzip2-${BZIP2_VER} &&
cp -v Makefile{,.orig} &&
sed -e 's@^\(all:.*\) test@\1@g' Makefile.orig > Makefile &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 CC="${CC}" AR="${AR}" RANLIB="${RANLIB}" &&
make PREFIX=/tools install &&
cd .. ||
exit 1

CHECK_VER=0.9.13
CHECK_URL="http://sourceforge.net/projects/check/files/check/${CHECK_VER}/check-${CHECK_VER}.tar.gz"
CHECK_NAME="${CHECK_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${CHECK_NAME} ] || wget ${CHECK_URL} -O ${CLFS_TARBALL_DIR}/${CHECK_NAME} || exit 1
echo "BUILDING check-${CHECK_VER}..."
rm -Rf check-${CHECK_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${CHECK_NAME} &&
cd check-${CHECK_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit 1

COREUTILS_VER=8.22
COREUTILS_URL="http://ftp.gnu.org/gnu/coreutils/coreutils-${COREUTILS_VER}.tar.xz"
COREUTILS_NAME="${COREUTILS_URL##*/}"
COREUTILS_PATCH_URL="http://patches.cross-lfs.org/dev/coreutils-${COREUTILS_VER}-noman-1.patch"
COREUTILS_PATCH_NAME="${COREUTILS_PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${COREUTILS_NAME} ] || wget ${COREUTILS_URL} -O ${CLFS_TARBALL_DIR}/${COREUTILS_NAME} || exit 1
[ -f ${CLFS_TARBALL_DIR}/${COREUTILS_PATCH_NAME} ] || wget ${COREUTILS_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${COREUTILS_PATCH_NAME} || exit 1
echo "BUILDING coreutils-${COREUTILS_VER}..."
rm -Rf coreutils-${COREUTILS_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${COREUTILS_NAME} &&
cd coreutils-${COREUTILS_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${COREUTILS_PATCH_NAME} &&
./configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --enable-install-program=hostname --cache-file=config.cache &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit 1

DIFFUTILS_VER=3.3
DIFFUTILS_URL="http://ftp.gnu.org/gnu/diffutils/diffutils-${DIFFUTILS_VER}.tar.xz"
DIFFUTILS_NAME="${DIFFUTILS_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${DIFFUTILS_NAME} ] || wget ${DIFFUTILS_URL} -O ${CLFS_TARBALL_DIR}/${DIFFUTILS_NAME} || exit 1
echo "BUILDING diffutils-${DIFFUTILS_VER}..."
rm -Rf diffutils-${DIFFUTILS_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${DIFFUTILS_NAME} &&
cd diffutils-${DIFFUTILS_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

FINDUTILS_VER=4.4.2
FINDUTILS_URL="http://ftp.gnu.org/gnu/findutils/findutils-${FINDUTILS_VER}.tar.gz"
FINDUTILS_NAME="${FINDUTILS_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${FINDUTILS_NAME} ] || wget ${FINDUTILS_URL} -O ${CLFS_TARBALL_DIR}/${FINDUTILS_NAME} || exit 1
echo "BUILDING findutils-${FINDUTILS_VER}..."
rm -Rf findutils-${FINDUTILS_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${FINDUTILS_NAME} &&
cd findutils-${FINDUTILS_VER} &&
echo "gl_cv_func_wcwidth_works=yes" > config.cache &&
echo "ac_cv_func_fnmatch_gnu=yes" >> config.cache &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} --cache-file=config.cache &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

echo "BUILDING file-${FILE_VER}..."
rm -Rf file-${FILE_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${FILE_NAME} &&
cd file-${FILE_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j $CORECOUNT -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

GAWK_VER=4.1.1
GAWK_URL="http://ftp.gnu.org/gnu/gawk/gawk-${GAWK_VER}.tar.gz"
GAWK_NAME="${GAWK_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GAWK_NAME} ] || wget ${GAWK_URL} -O ${CLFS_TARBALL_DIR}/${GAWK_NAME} || exit 1
echo "BUILDING gawk-${GAWK_VER}..."
rm -Rf gawk-${GAWK_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GAWK_NAME} &&
cd gawk-${GAWK_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

GETTEXT_VER=0.19
GETTEXT_URL="http://ftp.gnu.org/gnu/gettext/gettext-${GETTEXT_VER}.tar.gz"
GETTEXT_NAME="${GETTEXT_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GETTEXT_NAME} ] || wget ${GETTEXT_URL} -O ${CLFS_TARBALL_DIR}/${GETTEXT_NAME} || exit 1
echo "BUILDING gettext-tools-${GETTEXT_VER}..."
rm -Rf gettext-${GETTEXT_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GETTEXT_NAME} &&
cd gettext-${GETTEXT_VER}/gettext-tools &&
echo "gl_cv_func_wcwidth_works=yes" > config.cache &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} --disable-shared --cache-file=config.cache &&
make -C gnulib-lib -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make -C src -j ${CORECOUNT} -l ${CORECOUNT}.0 msgfmt msgmerge xgettext &&
cp -v src/{msgfmt,msgmerge,xgettext} /tools/bin &&
cd ../.. ||
exit $?

GREP_VER=2.19
GREP_URL="http://ftp.gnu.org/gnu/grep/grep-${GREP_VER}.tar.xz"
GREP_NAME="${GREP_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${GREP_NAME} ] || wget ${GREP_URL} -O ${CLFS_TARBALL_DIR}/${GREP_NAME} || exit 1
echo "BUILDING grep-${GREP_VER}..."
rm -Rf grep-${GREP_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GREP_NAME} &&
cd grep-${GREP_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} --without-included-regex &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

GZIP_VER=1.6
GZIP_URL="http://ftp.gnu.org/gnu/gzip/gzip-${GZIP_VER}.tar.gz"
GZIP_NAME="${GZIP_URL##*/}"
[ -f ${CLFS_TARBAL_DIR}/${GZIP_NAME} ] || wget ${GZIP_URL} -O ${CLFS_TARBALL_DIR}/${GZIP_NAME} || exit 1
echo "BUILDING gzip-${GZIP_VER}..."
rm -Rf gzip-${GZIP_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${GZIP_NAME} &&
cd gzip-${GZIP_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

MAKE_VER=4.0
MAKE_URL="http://ftp.gnu.org/gnu/make/make-${MAKE_VER}.tar.gz"
MAKE_NAME="${MAKE_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${MAKE_NAME} ] || wget ${MAKE_URL} -O ${CLFS_TARBALL_DIR}/${MAKE_NAME} || exit 1
echo "BUILDING make-${MAKE_VER}..."
rm -Rf make-${MAKE_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${MAKE_NAME} &&
cd make-${MAKE_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

PATCH_VER=2.7.1
PATCH_URL="http://ftp.gnu.org/gnu/patch/patch-${PATCH_VER}.tar.gz"
PATCH_NAME="${PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${PATCH_NAME} ] || wget ${PATCH_URL} -O ${CLFS_TARBALL_DIR}/${PATCH_NAME} || exit 1
echo "BUILDING patch-${PATCH_VER}..."
rm -Rf patch-${PATCH_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${PATCH_NAME} &&
cd patch-${PATCH_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

SED_VER=4.2.2
SED_URL="http://ftp.gnu.org/gnu/sed/sed-${SED_VER}.tar.gz" &&
SED_NAME="${SED_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${SED_NAME} ] || wget ${SED_URL} -O ${CLFS_TARBALL_DIR}/${SED_NAME} || exit 1
echo "BUILDING sed-${SED_VER}..."
rm -Rf sed-${SED_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${SED_NAME} &&
cd sed-${SED_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

TAR_VER=1.27.1
TAR_URL="http://ftp.gnu.org/gnu/tar/tar-${TAR_VER}.tar.gz"
TAR_NAME="${TAR_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${TAR_NAME} ] || wget ${TAR_URL} -O ${CLFS_TARBALL_DIR}/${TAR_NAME} || exit 1
echo "BUILDING tar-${TAR_VER}..."
rm -Rf tar-${TAR_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${TAR_NAME} &&
cd tar-${TAR_VER} ||
exit $?

cat > config.cache << EOF
gl_cv_func_wcwidth_works=yes
gl_cv_func_btowc_eof=yes
ac_cv_func_malloc_0_nonnull=yes
gl_cv_func_mbrtowc_incomplete_state=yes
gl_cv_func_mbrtowc_nul_retval=yes
gl_cv_func_mbrtowc_null_arg1=yes
gl_cv_func_mbrtowc_null_arg2=yes
gl_cv_func_mbrtowc_retval=yes
gl_cv_func_wcrtomb_retval=yes
EOF

./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} --cache-file=config.cache &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

TEXINFO_VER=5.2
TEXINFO_URL="http://ftp.gnu.org/gnu/texinfo/texinfo-${TEXINFO_VER}.tar.gz"
TEXINFO_NAME="${TEXINFO_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${TEXINFO_NAME} ] || wget ${TEXINFO_URL} -O ${CLFS_TARBALL_DIR}/${TEXINFO_NAME} || exit 1
echo "BUILDING texinfo-${TEXINFO_VER}..."
rm -Rf texinfo-${TEXINFO_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${TEXINFO_NAME} &&
cd texinfo-${TEXINFO_VER} &&
PERL=/usr/bin/perl ./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

UTIL_LINUX_VER=2.24.2
UTIL_LINUX_MAJMIN=`echo ${UTIL_LINUX_VER//./ } | awk '{ print $1 "." $2 }'`
UTIL_LINUX_URL="https://www.kernel.org/pub/linux/utils/util-linux/v${UTIL_LINUX_MAJMIN}/util-linux-${UTIL_LINUX_VER}.tar.gz"
UTIL_LINUX_NAME="${UTIL_LINUX_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${UTIL_LINUX_NAME} ] || wget ${UTIL_LINUX_URL} -O ${CLFS_TARBALL_DIR}/${UTIL_LINUX_NAME} || exit 1
echo "BUILDING util-linux-${UTIL_LINUX_VER}..."
rm -Rf util-linux-${UTIL_LINUX_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${UTIL_LINUX_NAME} &&
cd util-linux-${UTIL_LINUX_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} --disable-makeinstall-chown --disable-makeinstall-setuid &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

VIM_VER=7.4
VIM_URL="ftp://ftp.vim.org/pub/vim/unix/vim-${VIM_VER}.tar.bz2"
VIM_NAME="${VIM_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${VIM_NAME} ] || wget ${VIM_URL} -O ${CLFS_TARBALL_DIR}/${VIM_NAME} || exit 1

for VIM_PATCH_NUM in `seq 316`; do
    VIM_PATCH_NUM=`printf "%03d" $VIM_PATCH_NUM`
    VIM_PATCH_URL="${VIM_URL%/unix/*}/patches/${VIM_VER}/${VIM_VER}.${VIM_PATCH_NUM}"
    VIM_PATCH_NAME="vim-${VIM_PATCH_URL##*/}.patch"
    [ -f ${CLFS_TARBALL_DIR}/${VIM_PATCH_NAME} ] || wget ${VIM_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${VIM_PATCH_NAME} || exit 1
done

echo "BUILDING vim-${VIM_VER}..."
rm -Rf vim${VIM_VER//./} &&
tar xvf ${CLFS_TARBALL_DIR}/${VIM_NAME} &&
cd vim${VIM_VER//./} &&

for VIM_PATCH_NUM in `seq 316`; do
    VIM_PATCH_NUM=`printf "%03d" $VIM_PATCH_NUM`
    VIM_PATCH_NAME="vim-${VIM_VER}.${VIM_PATCH_NUM}.patch"
    echo "Applying patch ${VIM_PATCH_NAME}..."
    patch -p0 < ${CLFS_TARBALL_DIR}/${VIM_PATCH_NAME} || exit 1
done

cat > src/auto/config.cache << "EOF"
vim_cv_getcwd_broken=no
vim_cv_memmove_handles_overlap=yes
vim_cv_stat_ignores_slash=no
vim_cv_terminfo=yes
vim_cv_toupper_broken=no
vim_cv_tty_group=world
EOF

echo '#define SYS_VIMRC_FILE "/tools/etc/vimrc"' >> src/feature.h &&
./configure \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --prefix=/tools --enable-multibyte --enable-gui=no \
    --disable-gtktest --disable-xim --with-features=normal \
    --disable-gpm --without-x --disable-netbeans \
    --with-tlib=ncurses &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
ln -s vim /tools/bin/vi &&
cd .. ||
exit $?

cat > /tools/etc/vimrc << "EOF"
" Begin /tools/etc/vimrc

set nocompatible
set backspace=2
set ruler
syntax on

" End /tools/etc/vimrc
EOF

XZ_VER=5.0.5
XZ_URL="http://tukaani.org/xz/xz-${XZ_VER}.tar.gz"
XZ_NAME="${XZ_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${XZ_NAME} ] || wget ${XZ_URL} -O ${CLFS_TARBALL_DIR}/${XZ_NAME} || exit 1
echo "BUILDING xz-${XZ_VER}..."
rm -Rf xz-${XZ_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${XZ_NAME} &&
cd xz-${XZ_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

BOOT_SCRIPTS_VER="3.0-20140521"
BOOT_SCRIPTS_URL="http://cross-lfs.org/files/packages/git/boot-scripts-cross-lfs-${BOOT_SCRIPTS_VER}.tar.xz"
BOOT_SCRIPTS_NAME="${BOOT_SCRIPTS_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${BOOT_SCRIPTS_NAME} ] || wget ${BOOT_SCRIPTS_URL} -O ${CLFS_TARBALL_DIR}/${BOOT_SCRIPTS_NAME} || exit 1
echo "BUILDING boot-scripts-cross-lfs-${BOOT_SCRIPTS_VER}..."
rm -Rf boot-scripts-cross-lfs-${BOOT_SCRIPTS_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${BOOT_SCRIPTS_NAME} &&
cd boot-scripts-cross-lfs-${BOOT_SCRIPTS_VER} &&
make DESTDIR=/tools install-boot-scripts &&
echo UTC=1 > /tools/etc/sysconfig/clock &&
cd .. ||
exit $?

# FIXME: 1.42.10 doesn't cross-compile so well.
E2FSPROGS_VER=1.42.9
E2FSPROGS_URL="http://www.kernel.org/pub/linux/kernel/people/tytso/e2fsprogs/v${E2FSPROGS_VER}/e2fsprogs-${E2FSPROGS_VER}.tar.xz"
E2FSPROGS_NAME="${E2FSPROGS_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${E2FSPROGS_NAME} ] || wget ${E2FSPROGS_URL} -O ${CLFS_TARBALL_DIR}/${E2FSPROGS_NAME} || exit 1
echo "BUILDING e2fsprogs-${E2FSPROGS_VER}..."
rm -Rf e2fsprogs-${E2FSPROGS_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${E2FSPROGS_NAME} &&
cd e2fsprogs-${E2FSPROGS_VER} &&
mkdir -v build &&
cd build &&
../configure --prefix=/tools \
    --enable-elf-shlibs --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --disable-libblkid --disable-libuuid --disable-fsck \
    --disable-uuidd &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
make install-libs &&
cd ../.. ||
exit $?

KMOD_VER=17
KMOD_URL="http://www.kernel.org/pub/linux/utils/kernel/kmod/kmod-${KMOD_VER}.tar.xz"
KMOD_NAME="${KMOD_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${KMOD_NAME} ] || wget ${KMOD_URL} -O ${CLFS_TARBALL_DIR}/${KMOD_NAME} || exit 1
echo "BUILDING kmod-${KMOD_VER}..."
rm -Rf kmod-${KMOD_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${KMOD_NAME} &&
cd kmod-${KMOD_VER} &&
cp -v libkmod/libkmod.c{,.orig} &&
sed '/dirname_default_prefix /s@/lib/modules@/tools&@' \
    libkmod/libkmod.c.orig > libkmod/libkmod.c &&
./configure --prefix=/tools \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} --with-xz --with-zlib &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
ln -sfv kmod /tools/bin/lsmod ||
exit $?

for tool in depmod insmod modprobe modinfo rmmod; do
    ln -sv ../bin/kmod /tools/sbin/${tool} || exit $?
done

cd .. || exit $?

SHADOW_VER=4.2.1
SHADOW_URL="http://pkg-shadow.alioth.debian.org/releases/shadow-${SHADOW_VER}.tar.xz"
SHADOW_NAME="${SHADOW_URL##*/}"
SHADOW_PATCH_NAME=shadow-${SHADOW_VER}-vflg-1.patch
[ -f ${CLFS_TARBALL_DIR}/${SHADOW_NAME} ] || wget ${SHADOW_URL} -O ${CLFS_TARBALL_DIR}/${SHADOW_NAME} || exit 1
echo "BUILDING shadow-${SHADOW_VER}..."
rm -Rf shadow-${SHADOW_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${SHADOW_NAME} &&
cd shadow-${SHADOW_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${SHADOW_PATCH_NAME} &&
cp -v src/Makefile.in{,.orig} &&
sed -e 's/groups$(EXEEXT) //' -e 's/= nologin$(EXEEXT)/= /' \
    src/Makefile.in.orig > src/Makefile.in &&
./configure --prefix=/tools --disable-subordinate-ids \
    --build=${CLFS_HOST} --host=${CLFS_TARGET} &&
cp -v config.h{,.orig} &&
sed '/PASSWD_PROGRAM/s@/bin/passwd@/tools&@' config.h.orig > config.h &&
cp -v src/Makefile{,.orig} &&
sed 's/\(^suidu*bins = \).*/\1/' src/Makefile.orig > src/Makefile &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
cd .. ||
exit $?

SYSVINIT_VER=2.88dsf
SYSVINIT_URL="http://download.savannah.gnu.org/releases/sysvinit/sysvinit-${SYSVINIT_VER}.tar.bz2"
SYSVINIT_NAME="${SYSVINIT_URL##*/}"
SYSVINIT_PATCH_URL="http://patches.cross-lfs.org/dev/sysvinit-${SYSVINIT_VER}-tools_updates-1.patch"
SYSVINIT_PATCH_NAME="${SYSVINIT_PATCH_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${SYSVINIT_NAME} ] || wget ${SYSVINIT_URL} -O ${CLFS_TARBALL_DIR}/${SYSVINIT_NAME} || exit 1
[ -f ${CLFS_TARBALL_DIR}/${SYSVINIT_PATCH_NAME} ] || wget ${SYSVINIT_PATCH_URL} -O ${CLFS_TARBALL_DIR}/${SYSVINIT_PATCH_NAME} || exit 1
echo "BUILDING sysvinit-${SYSVINIT_VER}..."
rm -Rf sysvinit-${SYSVINIT_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${SYSVINIT_NAME} &&
cd sysvinit-${SYSVINIT_VER} &&
patch -Np1 -i ${CLFS_TARBALL_DIR}/${SYSVINIT_PATCH_NAME} &&
make -C src clobber &&
make -C src CC="${CC}" &&
make -C src ROOT=/tools install || exit $?

cat > /tools/etc/inittab << "EOF"
# Begin /tools/etc/inittab

id:3:initdefault:

si::sysinit:/tools/etc/rc.d/init.d/rc sysinit

l0:0:wait:/tools/etc/rc.d/init.d/rc 0
l1:S1:wait:/tools/etc/rc.d/init.d/rc 1
l2:2:wait:/tools/etc/rc.d/init.d/rc 2
l3:3:wait:/tools/etc/rc.d/init.d/rc 3
l4:4:wait:/tools/etc/rc.d/init.d/rc 4
l5:5:wait:/tools/etc/rc.d/init.d/rc 5
l6:6:wait:/tools/etc/rc.d/init.d/rc 6

ca:12345:ctrlaltdel:/tools/sbin/shutdown -t1 -a -r now

su:S016:once:/tools/sbin/sulogin

#c0:12345:respawn:/tools/sbin/agetty --noclear 115200 ttyPS0 vt100
c0:12345:respawn:/tools/sbin/agetty --noclear 115200 console vt100

# End /tools/etc/inittab
EOF

SAVERET=$?
[ $SAVERET -ne 0 ] && exit $SAVERET

cd .. || exit $?

EUDEV_VER=1.7
EUDEV_URL="ftp://mirror.ovh.net/gentoo-distfiles/distfiles/eudev-${EUDEV_VER}.tar.gz"
EUDEV_NAME="${EUDEV_URL##*/}"
[ -f ${CLFS_TARBALL_DIR}/${EUDEV_NAME} ] || wget ${EUDEV_URL} -O ${CLFS_TARBALL_DIR}/${EUDEV_NAME} || exit 1
echo "BUILDING eudev-${EUDEV_VER}..."
rm -Rf eudev-${EUDEV_VER} &&
tar xvf ${CLFS_TARBALL_DIR}/${EUDEV_NAME} &&
cd eudev-${EUDEV_VER} &&
./configure --prefix=/tools --build=${CLFS_HOST} --host=${CLFS_TARGET} \
    --disable-introspection --disable-gtk-doc-html \
    --disable-gudev --disable-keymap --with-firmware-path=/tools/lib/firmware \
    --enable-libkmod &&
make -j ${CORECOUNT} -l ${CORECOUNT}.0 &&
make install &&
install -dv /tools/lib/firmware &&
echo "# dummy, so that network is once again on eth*" > /tools/etc/udev/rules.d/80-net-name-slot.rules &&
cd .. ||
exit $?

echo "BUILDING parallella-kernel-${LINUX_VER}..."
rm -Rf parallella-linux-adi-xcomm_zynq &&
tar xvf ${CLFS_TARBALL_DIR}/${LINUX_NAME} &&
cd parallella-linux-adi-xcomm_zynq &&
make mrproper &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- parallella_defconfig &&
cp ${CLFS_TARBALL_DIR}/linux-parallella-${LINUX_VER}.config .config &&
yes "" | make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- oldconfig &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- LOADADDR=0x8000 -j ${CORECOUNT} -l ${CORECOUNT}.0 uImage &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- LOADADDR=0x8000 -j ${CORECOUNT} -l ${CORECOUNT}.0 modules &&
make ARCH=arm CROSS_COMPILE=${CLFS_TARGET}- LOADADDR=0x8000 INSTALL_MOD_PATH=${CLFS} modules_install &&
install -dv -m 0755 ${CLFS}/boot &&
install -v -m 0644 arch/arm/boot/uImage ${CLFS}/boot/uImage-${LINUX_VER} &&
cd .. ||
exit $?

echo "Creating base directories, symlinks, and device inodes..."
mkdir -pv ${CLFS}/{bin,boot,dev,{etc/,}opt,home,lib/firmware,mnt} &&
mkdir -pv ${CLFS}/mnt/BOOT &&
mkdir -pv ${CLFS}/{proc,media/{floppy,cdrom},run/{,shm},sbin,srv,sys} &&
mkdir -pv ${CLFS}/var/{lock,log,mail,spool} &&
mkdir -pv ${CLFS}/var/{opt,cache,lib/{misc,locate},local} &&
install -dv -m 0750 ${CLFS}/root &&
install -dv -m 1777 ${CLFS}{/var,}/tmp &&
ln -sv ../run ${CLFS}/var/run &&
mkdir -pv ${CLFS}/usr/{,local/}{bin,include,lib,sbin,src} &&
mkdir -pv ${CLFS}/usr/{,local/}share/{doc,info,locale,man} &&
mkdir -pv ${CLFS}/usr/{,local/}share/{misc,terminfo,zoneinfo} &&
mkdir -pv ${CLFS}/usr/{,local/}share/man/man{1,2,3,4,5,6,7,8} &&
ln -sv /tools/bin/{bash,cat,echo,grep,login,pwd,stty} ${CLFS}/bin &&
ln -sv /tools/bin/file ${CLFS}/usr/bin &&
ln -sv /tools/lib/libgcc_s.so{,.1} ${CLFS}/usr/lib &&
ln -sv /tools/lib/libstdc++.so{.6,} ${CLFS}/usr/lib &&
sed -e 's/tools/usr/' /tools/lib/libstdc++.la > ${CLFS}/usr/lib/libstdc++.la &&
ln -sv bash ${CLFS}/bin/sh &&
ln -sv /tools/sbin/init ${CLFS}/sbin &&
ln -sv /tools/etc/{login.{access,defs},limits} ${CLFS}/etc &&
ln -sv /proc/self/mounts ${CLFS}/etc/mtab ||
exit $?

echo "Creating /etc/passwd and /etc/group..."
cat > ${CLFS}/etc/passwd << "EOF"
root::0:0:root:/root:/bin/bash
bin:x:1:1:/bin:/bin/false
daemon:x:2:6:/sbin:/bin/false
messagebus:x:27:27:D-Bus Message Daemon User:/dev/null:/bin/false
nobody:x:65534:65533:Unprivileged User:/dev/null:/bin/false
EOF

RETVAL=$?
[ "$RETVAL" != "0" ] && exit $RETVAL

cat > ${CLFS}/etc/group << "EOF"
root:x:0:
bin:x:1:
sys:x:2:
kmem:x:3:
tty:x:5:
tape:x:4:
daemon:x:6:
floppy:x:7:
disk:x:8:
lp:x:9:
dialout:x:10:
audio:x:11:
video:x:12:
utmp:x:13:
usb:x:14:
cdrom:x:15:
adm:x:16:
messagebus:x:27:
systemd-journal:x:28:
mail:x:30:
wheel:x:39:
nogroup:x:65533:
EOF

RETVAL=$?
[ "$RETVAL" != "0" ] && exit $RETVAL

echo "Creating /etc/fstab..."

cat > ${CLFS}/etc/fstab << "EOF"
# Begin /etc/fstab

# file system  mount-point  type   options          dump  fsck
#                                                         order

/dev/mmcblk0p2 /            ext4   defaults         1     1
/dev/mmcblk0p1 /mnt/BOOT    vfat   noauto           0     0
devpts         /dev/pts     devpts gid=5,mode=620   0     0
shm            /dev/shm     tmpfs  defaults         0     0
tmp            /tmp         tmpfs  defaults         0     0
run            /var/run     tmpfs  defaults         0     0
proc           /proc        procfs defaults         0     0
sys            /sys         sysfs  defaults         0     0

# End /etc/fstab
EOF

RETVAL=$?
[ "$RETVAL" != "0" ] && exit $RETVAL

echo "Creating /root/.bash_profile..."
cat > ${CLFS}/root/.bash_profile << "EOF"
set +h
PS1='\u:\w\$ '
LC_ALL=POSIX
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/tools/bin:/tools/sbin
export LC_ALL PATH PS1
EOF

RETVAL=$?
[ "$RETVAL" != "0" ] && exit $RETVAL

echo "Setting permissions..."
$SU_CMD find ${CLFS} -owner ${CLFS_USER} -exec chown 0:0 {} \;

exit $?

